# Copyright 2023 Avaiga Private Limited
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.

from functools import wraps
from unittest.mock import MagicMock, patch

from src.taipy.rest.api.middlewares._middleware import _middleware


def mock_enterprise_middleware(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        return f(*args, **kwargs)

    return wrapper


@patch("src.taipy.rest.api.middlewares._middleware._using_enterprise")
@patch("src.taipy.rest.api.middlewares._middleware._enterprise_middleware")
def test_enterprise_middleware_applied_when_enterprise_is_installed(
    enterprise_middleware: MagicMock, using_enterprise: MagicMock
):
    enterprise_middleware.return_value = mock_enterprise_middleware
    using_enterprise.return_value = True

    @_middleware
    def f():
        return "f"

    rv = f()
    assert rv == "f"
    using_enterprise.assert_called_once()
    enterprise_middleware.assert_called_once()


@patch("src.taipy.rest.api.middlewares._middleware._using_enterprise")
@patch("src.taipy.rest.api.middlewares._middleware._enterprise_middleware")
def test_enterprise_middleware_not_applied_when_enterprise_is_not_installed(
    enterprise_middleware: MagicMock, using_enterprise: MagicMock
):
    enterprise_middleware.return_value = mock_enterprise_middleware
    using_enterprise.return_value = False

    @_middleware
    def f():
        return "f"

    rv = f()
    assert rv == "f"
    using_enterprise.assert_called_once()
    enterprise_middleware.assert_not_called()



# Copyright 2023 Avaiga Private Limited
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.

import json
from typing import Dict

from flask import url_for


def create_and_submit_scenario(config_id: str, client) -> Dict:
    response = client.post(url_for("api.scenarios", config_id=config_id))
    assert response.status_code == 201

    scenario = response.json.get("scenario")
    assert (set(scenario) - set(json.load(open("tests/json/expected/scenario.json")))) == set()

    response = client.post(url_for("api.scenario_submit", scenario_id=scenario.get("id")))
    assert response.status_code == 200

    return scenario


def get(url, name, client) -> Dict:
    response = client.get(url)
    returned_data = response.json.get(name)

    assert (set(returned_data) - set(json.load(open(f"tests/json/expected/{name}.json")))) == set()

    return returned_data


def get_assert_status(url, client, status_code) -> None:
    response = client.get(url)
    assert response.status_code == status_code


def get_all(url, expected_quantity, client):
    response = client.get(url)

    assert len(response.json) == expected_quantity


def delete(url, client):
    response = client.delete(url)

    assert response.status_code == 200


def test_end_to_end(client, setup_end_to_end):
    # Create Scenario: Should also create all of its dependencies(pipelines, tasks, datanodes, etc)
    scenario = create_and_submit_scenario("scenario", client)

    # Get other models and verify if they return the necessary fields
    cycle = get(url_for("api.cycle_by_id", cycle_id=scenario.get("cycle")), "cycle", client)
    pipeline = get(
        url_for("api.pipeline_by_id", pipeline_id=scenario.get("pipelines")[0]),
        "pipeline",
        client,
    )
    task = get(url_for("api.task_by_id", task_id=pipeline.get("tasks")[0]), "task", client)
    datanode = get(
        url_for("api.datanode_by_id", datanode_id=task.get("input_ids")[0]),
        "datanode",
        client,
    )
    # Get All
    get_all(url_for("api.scenarios"), 1, client)
    get_all(url_for("api.cycles"), 1, client)
    get_all(url_for("api.pipelines"), 1, client)
    get_all(url_for("api.tasks"), 2, client)
    get_all(url_for("api.datanodes"), 5, client)
    get_all(url_for("api.jobs"), 2, client)

    # Delete entities
    delete(url_for("api.cycle_by_id", cycle_id=cycle.get("id")), client)
    delete(url_for("api.pipeline_by_id", pipeline_id=pipeline.get("id")), client)
    delete(url_for("api.task_by_id", task_id=task.get("id")), client)
    delete(url_for("api.datanode_by_id", datanode_id=datanode.get("id")), client)

    # Check status code
    # Non-existing entities should return 404
    get_assert_status(url_for("api.cycle_by_id", cycle_id=9999999), client, 404)
    get_assert_status(url_for("api.scenario_by_id", scenario_id=9999999), client, 404)
    get_assert_status(url_for("api.pipeline_by_id", pipeline_id=9999999), client, 404)
    get_assert_status(url_for("api.task_by_id", task_id=9999999), client, 404)
    get_assert_status(url_for("api.datanode_by_id", datanode_id=9999999), client, 404)



# Copyright 2023 Avaiga Private Limited
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.

"""Extensions registry

All extensions here are used as singletons and
initialized in application factory
"""
from flask_marshmallow import Marshmallow
from flask_migrate import Migrate
from passlib.context import CryptContext

from .commons.apispec import APISpecExt

ma = Marshmallow()
migrate = Migrate()
apispec = APISpecExt()
pwd_context = CryptContext(schemes=["pbkdf2_sha256"], deprecated="auto")



# Copyright 2023 Avaiga Private Limited
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.

from unittest import mock

from flask import url_for


def test_get_cycle(client, default_cycle):
    # test 404
    cycle_url = url_for("api.cycle_by_id", cycle_id="foo")
    rep = client.get(cycle_url)
    assert rep.status_code == 404

    with mock.patch("taipy.core.cycle._cycle_manager._CycleManager._get") as manager_mock:
        manager_mock.return_value = default_cycle

        # test get_cycle
        rep = client.get(url_for("api.cycle_by_id", cycle_id="foo"))
        assert rep.status_code == 200


def test_delete_cycle(client):
    # test 404
    cycle_url = url_for("api.cycle_by_id", cycle_id="foo")
    rep = client.get(cycle_url)
    assert rep.status_code == 404

    with mock.patch("taipy.core.cycle._cycle_manager._CycleManager._delete"), mock.patch(
        "taipy.core.cycle._cycle_manager._CycleManager._get"
    ):
        # test get_cycle
        rep = client.delete(url_for("api.cycle_by_id", cycle_id="foo"))
        assert rep.status_code == 200


def test_create_cycle(client, cycle_data):
    # without config param
    cycles_url = url_for("api.cycles")
    data = {"bad": "data"}
    rep = client.post(cycles_url, json=data)
    assert rep.status_code == 400

    rep = client.post(cycles_url, json=cycle_data)
    assert rep.status_code == 201


def test_get_all_cycles(client, create_cycle_list):
    cycles_url = url_for("api.cycles")
    rep = client.get(cycles_url)
    assert rep.status_code == 200

    results = rep.get_json()
    assert len(results) == 10



# Copyright 2023 Avaiga Private Limited
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.

from unittest import mock

from flask import url_for


def test_get_job(client, default_job):
    # test 404
    user_url = url_for("api.job_by_id", job_id="foo")
    rep = client.get(user_url)
    assert rep.status_code == 404

    with mock.patch("taipy.core.job._job_manager._JobManager._get") as manager_mock:
        manager_mock.return_value = default_job

        # test get_job
        rep = client.get(url_for("api.job_by_id", job_id="foo"))
        assert rep.status_code == 200


def test_delete_job(client):
    # test 404
    user_url = url_for("api.job_by_id", job_id="foo")
    rep = client.get(user_url)
    assert rep.status_code == 404

    with mock.patch("taipy.core.job._job_manager._JobManager._delete"), mock.patch(
        "taipy.core.job._job_manager._JobManager._get"
    ):
        # test get_job
        rep = client.delete(url_for("api.job_by_id", job_id="foo"))
        assert rep.status_code == 200


def test_create_job(client, default_task_config):
    # without config param
    jobs_url = url_for("api.jobs")
    rep = client.post(jobs_url)
    assert rep.status_code == 400

    with mock.patch("src.taipy.rest.api.resources.job.JobList.fetch_config") as config_mock:
        config_mock.return_value = default_task_config
        jobs_url = url_for("api.jobs", task_id="foo")
        rep = client.post(jobs_url)
        assert rep.status_code == 201


def test_get_all_jobs(client, create_job_list):
    jobs_url = url_for("api.jobs")
    rep = client.get(jobs_url)
    assert rep.status_code == 200

    results = rep.get_json()
    assert len(results) == 10


def test_cancel_job(client, default_job):
    # test 404
    from taipy.core._orchestrator._orchestrator_factory import _OrchestratorFactory

    _OrchestratorFactory._build_orchestrator()
    _OrchestratorFactory._build_dispatcher()

    user_url = url_for("api.job_cancel", job_id="foo")
    rep = client.post(user_url)
    assert rep.status_code == 404

    with mock.patch("taipy.core.job._job_manager._JobManager._get") as manager_mock:
        manager_mock.return_value = default_job

        # test get_job
        rep = client.post(url_for("api.job_cancel", job_id="foo"))
        assert rep.status_code == 200



# Copyright 2023 Avaiga Private Limited
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.

from .cycle import CycleList, CycleResource
from .datanode import DataNodeList, DataNodeReader, DataNodeResource, DataNodeWriter
from .job import JobList, JobResource, JobExecutor
from .pipeline import PipelineExecutor, PipelineList, PipelineResource
from .scenario import ScenarioExecutor, ScenarioList, ScenarioResource
from .task import TaskExecutor, TaskList, TaskResource

__all__ = [
    "DataNodeResource",
    "DataNodeList",
    "DataNodeReader",
    "DataNodeWriter",
    "TaskList",
    "TaskResource",
    "TaskExecutor",
    "PipelineList",
    "PipelineResource",
    "PipelineExecutor",
    "ScenarioList",
    "ScenarioResource",
    "ScenarioExecutor",
    "CycleResource",
    "CycleList",
    "JobResource",
    "JobList",
    "JobExecutor",
]



import json
from typing import Any, Union
from datetime import datetime
from enum import Enum



Json = Union[dict, list, str, int, float, bool, None]

class _CustomEncoder(json.JSONEncoder):
    def default(self, o: Any) -> Json:
        if isinstance(o, Enum):
            result = o.value
        elif isinstance(o, datetime):
            result = {"__type__": "Datetime", "__value__": o.isoformat()}
        else:
            result = json.JSONEncoder.default(self, o)
        return result
